package at.ac.uibk.gitsearch.service.dto;

import java.util.List;
import java.util.stream.Collectors;

import at.ac.uibk.gitsearch.service.dto.UserProvidedMetadataDTO.ExerciseType;

public class SearchInputMetadataDTO {
    private String programmingLanguage;
    private String keyword;
    private List<String> naturalLanguage;
    private String license;
    private String author;
    private List<ExerciseType> types;
    private String format;

    public SearchInputMetadataDTO() {
    }


    public SearchInputMetadataDTO(String programmingLanguage,
                                  String keyword,
                                  List<String> naturalLanguage,
                                  String license,
                                  String author,
                                  String format) {
        this.programmingLanguage = mapEmptyString(programmingLanguage);
        this.keyword = mapEmptyString(keyword);
        this.license = mapEmptyString(license);
        this.author = mapEmptyString(author);
        this.naturalLanguage = naturalLanguage;
        this.format = mapEmptyString(format);
    }

    private static List<String> mapEmptyString(List<String> str) {
    	if(str==null) return null;
    	return str.stream().map(s -> "".equals(s) ? null : s).collect(Collectors.toList());
    }

    private static String mapEmptyString(String str) {
        return "".equals(str) ? null : str;
    }

    public String getFormat() {
        return format;
    }

    public void setFormat(String format) {
        this.format = format;
    }

    public boolean hasMetadataInput() {
        return !(programmingLanguage == null
            && keyword == null
            && author == null
            && license == null);
    }

    public String getProgrammingLanguage() {
        return programmingLanguage;
    }

    public void setProgrammingLanguage(String programmingLanguage) {
        this.programmingLanguage = mapEmptyString(programmingLanguage);
    }

    public String getKeyword() {
        return keyword;
    }

    public void setKeyword(String keyword) {
        this.keyword = mapEmptyString(keyword);
    }

    public List<String> getNaturalLanguage() {
        return naturalLanguage;
    }

    public String getNaturalLanguagesAsString() {
    	return naturalLanguage==null?"null":
    		naturalLanguage.stream().collect(Collectors.joining(", "));
    }

    public void setNaturalLanguage(List<String> naturalLanguage) {
        this.naturalLanguage = mapEmptyString(naturalLanguage);
    }

    public String getLicense() {
        return license;
    }

    public void setLicense(String license) {
        this.license = mapEmptyString(license);
    }

    public String getAuthor() {
        return author;
    }

    public void setAuthor(String author) {
        this.author = mapEmptyString(author);
    }

    /**
	 * @return the types
	 */
	public List<ExerciseType> getTypes() {
		return types;
	}

	/**
	 * @param types the types to set
	 */
	public void setTypes(List<ExerciseType> types) {
		this.types = types;
	}

	@Override
    public String toString() {
        return "programmingLanguage: " + this.programmingLanguage
            + ", keyword: " + this.keyword
            + ", naturalLanguage: " + getNaturalLanguagesAsString()
            + ", license: " + this.license
            + ", author: " + this.author
            + ", format: " + this.format;
    }
}
