package at.ac.uibk.gitsearch.web.rest;

import at.ac.uibk.gitsearch.service.GitlabService;
import org.gitlab4j.api.GitLabApiException;
import org.gitlab4j.api.models.Group;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.naming.AuthenticationException;
import java.nio.file.attribute.GroupPrincipal;
import java.util.List;

@RestController
@RequestMapping("/api/gitlab")
public class GitlabResource {

    private final Logger log = LoggerFactory.getLogger(GitlabResource.class);

    private final GitlabService gitlabService;

    public GitlabResource(GitlabService gitlabService) {
        this.gitlabService = gitlabService;
    }

    /**
     * GET /gitlab/groups : Retrieve Gitlab groups of user
     * @return List of Gitlab groups
     * @throws GitLabApiException if an error occurs
     */
    @GetMapping("/groups")
    public ResponseEntity<List<Group>> getUserGroups() throws GitLabApiException {
        try {
            return ResponseEntity.ok(gitlabService.getGroups()); // mocked below TODO remove
        } catch (AuthenticationException ae) {
            return ResponseEntity.status(401).body(null);
        }
        // return ResponseEntity.ok("[{\"id\":62,\"name\":\"C-Exercises\",\"path\":\"c-exercises\",\"description\":\"\",\"visibility\":\"private\",\"lfsEnabled\":true,\"avatarUrl\":null,\"webUrl\":\"https://sharing.codeability-austria.uibk.ac.at/groups/sharing/university-innsbruck/c-exercises\",\"requestAccessEnabled\":true,\"fullName\":\"sharing / university-innsbruck / C-Exercises\",\"fullPath\":\"sharing/university-innsbruck/c-exercises\",\"parentId\":38,\"sharedRunnersMinutesLimit\":null,\"statistics\":null,\"projects\":null,\"sharedProjects\":null,\"createdAt\":\"2021-03-31T20:14:10.547+0000\",\"markedForDeletionOn\":null},{\"id\":65,\"name\":\"EidP-21S\",\"path\":\"eidp-21s\",\"description\":\"\",\"visibility\":\"private\",\"lfsEnabled\":true,\"avatarUrl\":null,\"webUrl\":\"https://sharing.codeability-austria.uibk.ac.at/groups/sharing/university-innsbruck/c-exercises/eidp-21s\",\"requestAccessEnabled\":true,\"fullName\":\"sharing / university-innsbruck / C-Exercises / EidP-21S\",\"fullPath\":\"sharing/university-innsbruck/c-exercises/eidp-21s\",\"parentId\":62,\"sharedRunnersMinutesLimit\":null,\"statistics\":null,\"projects\":null,\"sharedProjects\":null,\"createdAt\":\"2021-04-03T09:58:01.255+0000\",\"markedForDeletionOn\":null},{\"id\":56,\"name\":\"Vienna Universityof Technology\",\"path\":\"vienna-universityof-technology\",\"description\":\"\",\"visibility\":\"private\",\"lfsEnabled\":true,\"avatarUrl\":null,\"webUrl\":\"https://sharing.codeability-austria.uibk.ac.at/groups/sharing/vienna-universityof-technology\",\"requestAccessEnabled\":true,\"fullName\":\"sharing / Vienna Universityof Technology\",\"fullPath\":\"sharing/vienna-universityof-technology\",\"parentId\":35,\"sharedRunnersMinutesLimit\":null,\"statistics\":null,\"projects\":null,\"sharedProjects\":null,\"createdAt\":\"2020-12-15T16:13:54.054+0000\",\"markedForDeletionOn\":null},{\"id\":66,\"name\":\"accessTests\",\"path\":\"accesstests\",\"description\":\"Just some projects for access testing\",\"visibility\":\"private\",\"lfsEnabled\":true,\"avatarUrl\":null,\"webUrl\":\"https://sharing.codeability-austria.uibk.ac.at/groups/sharing/university-innsbruck/java/accesstests\",\"requestAccessEnabled\":false,\"fullName\":\"sharing / university-innsbruck / java / accessTests\",\"fullPath\":\"sharing/university-innsbruck/java/accesstests\",\"parentId\":39,\"sharedRunnersMinutesLimit\":null,\"statistics\":null,\"projects\":null,\"sharedProjects\":null,\"createdAt\":\"2021-04-09T09:01:30.892+0000\",\"markedForDeletionOn\":null},{\"id\":70,\"name\":\"collection_tests\",\"path\":\"collection_tests\",\"description\":\"\",\"visibility\":\"private\",\"lfsEnabled\":true,\"avatarUrl\":null,\"webUrl\":\"https://sharing.codeability-austria.uibk.ac.at/groups/sharing/health-check-tests/collection_tests\",\"requestAccessEnabled\":true,\"fullName\":\"sharing / health-check-tests / collection_tests\",\"fullPath\":\"sharing/health-check-tests/collection_tests\",\"parentId\":52,\"sharedRunnersMinutesLimit\":null,\"statistics\":null,\"projects\":null,\"sharedProjects\":null,\"createdAt\":\"2021-06-09T12:31:59.973+0000\",\"markedForDeletionOn\":null},{\"id\":40,\"name\":\"general\",\"path\":\"general\",\"description\":\"\",\"visibility\":\"public\",\"lfsEnabled\":true,\"avatarUrl\":null,\"webUrl\":\"https://sharing.codeability-austria.uibk.ac.at/groups/sharing/university-innsbruck/java/general\",\"requestAccessEnabled\":true,\"fullName\":\"sharing / university-innsbruck / java / general\",\"fullPath\":\"sharing/university-innsbruck/java/general\",\"parentId\":39,\"sharedRunnersMinutesLimit\":null,\"statistics\":null,\"projects\":null,\"sharedProjects\":null,\"createdAt\":\"2020-08-31T16:08:08.233+0000\",\"markedForDeletionOn\":null},{\"id\":52,\"name\":\"health-check-tests\",\"path\":\"health-check-tests\",\"description\":\"\",\"visibility\":\"private\",\"lfsEnabled\":true,\"avatarUrl\":null,\"webUrl\":\"https://sharing.codeability-austria.uibk.ac.at/groups/sharing/health-check-tests\",\"requestAccessEnabled\":true,\"fullName\":\"sharing / health-check-tests\",\"fullPath\":\"sharing/health-check-tests\",\"parentId\":35,\"sharedRunnersMinutesLimit\":null,\"statistics\":null,\"projects\":null,\"sharedProjects\":null,\"createdAt\":\"2020-09-10T13:54:44.308+0000\",\"markedForDeletionOn\":null},{\"id\":39,\"name\":\"java\",\"path\":\"java\",\"description\":\"\",\"visibility\":\"public\",\"lfsEnabled\":true,\"avatarUrl\":null,\"webUrl\":\"https://sharing.codeability-austria.uibk.ac.at/groups/sharing/university-innsbruck/java\",\"requestAccessEnabled\":true,\"fullName\":\"sharing / university-innsbruck / java\",\"fullPath\":\"sharing/university-innsbruck/java\",\"parentId\":38,\"sharedRunnersMinutesLimit\":null,\"statistics\":null,\"projects\":null,\"sharedProjects\":null,\"createdAt\":\"2020-08-31T16:07:47.824+0000\",\"markedForDeletionOn\":null},{\"id\":68,\"name\":\"modelling\",\"path\":\"modelling\",\"description\":\"\",\"visibility\":\"private\",\"lfsEnabled\":true,\"avatarUrl\":null,\"webUrl\":\"https://sharing.codeability-austria.uibk.ac.at/groups/sharing/university-innsbruck/modelling\",\"requestAccessEnabled\":true,\"fullName\":\"sharing / university-innsbruck / modelling\",\"fullPath\":\"sharing/university-innsbruck/modelling\",\"parentId\":38,\"sharedRunnersMinutesLimit\":null,\"statistics\":null,\"projects\":null,\"sharedProjects\":null,\"createdAt\":\"2021-04-12T07:20:36.312+0000\",\"markedForDeletionOn\":null},{\"id\":35,\"name\":\"sharing\",\"path\":\"sharing\",\"description\":\"\",\"visibility\":\"public\",\"lfsEnabled\":true,\"avatarUrl\":null,\"webUrl\":\"https://sharing.codeability-austria.uibk.ac.at/groups/sharing\",\"requestAccessEnabled\":true,\"fullName\":\"sharing\",\"fullPath\":\"sharing\",\"parentId\":null,\"sharedRunnersMinutesLimit\":null,\"statistics\":null,\"projects\":null,\"sharedProjects\":null,\"createdAt\":\"2020-08-27T11:47:06.509+0000\",\"markedForDeletionOn\":null},{\"id\":38,\"name\":\"university-innsbruck\",\"path\":\"university-innsbruck\",\"description\":\"\",\"visibility\":\"public\",\"lfsEnabled\":true,\"avatarUrl\":null,\"webUrl\":\"https://sharing.codeability-austria.uibk.ac.at/groups/sharing/university-innsbruck\",\"requestAccessEnabled\":true,\"fullName\":\"sharing / university-innsbruck\",\"fullPath\":\"sharing/university-innsbruck\",\"parentId\":35,\"sharedRunnersMinutesLimit\":null,\"statistics\":null,\"projects\":null,\"sharedProjects\":null,\"createdAt\":\"2020-08-31T16:07:13.587+0000\",\"markedForDeletionOn\":null}]");
    }

    /**
     * GET /gitlab/subgroups/{group} : Retrieve subgroups of group
     * @param groupId ID of group to retrieve its subgroups
     * @return List of subgroups of given group
     * @throws GitLabApiException if an error occurs
     */
    @GetMapping("/subgroups/{groupId}")
    public ResponseEntity<List<Group>> getSubGroups(@PathVariable("groupId") int groupId) throws GitLabApiException {
        return ResponseEntity.ok(gitlabService.getSubGroups(groupId));
    }
}
