package at.ac.uibk.gitsearch.repository.search;

import java.io.IOException;
import java.util.List;

import javax.ws.rs.NotFoundException;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.node.NodeValidationException;
import static org.hamcrest.Matchers.*;
import static org.junit.Assert.*;

import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;

import at.ac.uibk.gitsearch.GitsearchApp;
import at.ac.uibk.gitsearch.repository.search.testESService.ElasticSearchTestServerConfiguration;
import at.ac.uibk.gitsearch.service.dto.AutoCompleteEntry;
import at.ac.uibk.gitsearch.service.dto.SearchResultDTO;

@SpringBootTest(classes = GitsearchApp.class)
public class MetaDataRepositoryIT {

	@SuppressWarnings("unused")
	private static final Logger LOGGER = LogManager.getLogger(MetaDataRepositoryIT.class);

	@Autowired
	RestHighLevelClient elasticsearchTestClient;

	@BeforeAll
	public static void setUpESServer() throws IOException, NodeValidationException {
		ElasticSearchTestServerConfiguration.getTestInstance().startTestNode();
	}

	@BeforeEach
	public void setUpIndex() {
	}

	@Autowired
	private MetaDataRepository metaDataRepository;

	@Test
	public void testKeywordAutocompletion() throws IOException {
		final List<AutoCompleteEntry> keywordsAutoComplete = metaDataRepository.getKeywordsAutoComplete("Jav");
		assertThat(keywordsAutoComplete, contains(hasProperty("target", is("Java"))));
	}

	@Test
	public void testCreatorAutocompletion() throws IOException {
		final List<AutoCompleteEntry> creatorAutoComplete = metaDataRepository.getCreatorAutoComplete("Pod");
		assertThat(creatorAutoComplete, contains(hasProperty("target", is("Stefan Podlipnig"))));
		final List<AutoCompleteEntry> creatorAutoComplete2 = metaDataRepository.getCreatorAutoComplete("Po");
		assertThat(creatorAutoComplete2, contains(hasProperty("target", is("Stefan Podlipnig"))));
	}

	@Test
	public void testContributorAutocompletion() throws IOException {
		final List<AutoCompleteEntry> contributorAutoComplete = metaDataRepository.getContributorAutoComplete("Bast");
		assertThat(contributorAutoComplete, contains(hasProperty("target", is("Daniel Bastta"))));
	}

	@Test
	public void testProgrammingLanguageAutocompletion() throws IOException {
		final List<AutoCompleteEntry> plAutoComplete = metaDataRepository.getProgrammingLanguageAutoComplete("Ja");
		assertThat(plAutoComplete, contains(hasProperty("target", is("Java"))));
	}

	@Test
	public void getByExerciseId() throws IOException {
		// This test assumes that the test entries have consecutive ids
		for (int i : new int[] { 1, 2, 3 }) {
			final SearchResultDTO exerciseById = metaDataRepository.getExerciseById(i + "");
			assertNotNull("should be found:", exerciseById);
			assertEquals("Id should be the same", i + "", exerciseById.getExerciseId());
		}
	}

	@Test()
	public void getByExerciseIdNegative() throws IOException {
		// This test assumes that the test entries have consecutive ids
		String exerciseId = "999999"; // should not exist
		Assertions.assertThrows(NotFoundException.class, () -> {
			metaDataRepository.getExerciseById(exerciseId);},
				"Exercise for id " + exerciseId + " should not exist:");
	}
}
