package at.ac.uibk.gitsearch.service.dto;

import java.lang.reflect.InvocationTargetException;

import org.junit.Assert;
import org.junit.jupiter.api.Test;

import at.ac.uibk.gitsearch.config.ApplicationProperties.DeploymentInfo;
import at.ac.uibk.gitsearch.service.MailService;
import at.ac.uibk.gitsearch.service.dto.SearchResultDTO.GitProject;
import at.ac.uibk.gitsearch.service.dto.SearchResultsDTO.GitProjectVisibility;
import at.ac.uibk.gitsearch.service.dto.UserProvidedMetadataDTO.ExerciseType;
import at.ac.uibk.gitsearch.service.dto.UserProvidedMetadataDTO.Person;
import at.ac.uibk.gitsearch.testingUtilities.PropertiesTester;
import nl.jqno.equalsverifier.EqualsVerifier;
import nl.jqno.equalsverifier.Warning;

/**
 * Integration tests for {@link MailService}.
 */
public class VariousDTOTest {

	PropertiesTester propertiesTester = new PropertiesTester();

	@Test
	public void testUserProvidedMetadataDTO() throws IllegalAccessException, InvocationTargetException {
		
		
		propertiesTester.testProperties(UserProvidedMetadataDTO.class);
		UserProvidedMetadataDTO testUPMD = new UserProvidedMetadataDTO();
		UserProvidedMetadataDTO testUPMDclone = new UserProvidedMetadataDTO(testUPMD);
		Assert.assertEquals("clone should be equas", testUPMD, testUPMDclone);
		EqualsVerifier.forClass(UserProvidedMetadataDTO.class).suppress(Warning.NONFINAL_FIELDS).verify();

	}

	@Test
	public void testSearchResultsDTO() throws IllegalAccessException, InvocationTargetException {
		propertiesTester.testProperties(SearchResultsDTO.class);
		propertiesTester.testProperties(SearchResultDTO.class);
		propertiesTester.testProperties(Person.class);
		propertiesTester.testProperties(GitProject.class);
		
		// just for test coverage
		SearchResultsDTO srDTO = new SearchResultsDTO(null, 0, 0);
	}
	
	@Test
	public void testSearchResultsEnums() throws IllegalAccessException, InvocationTargetException {
		for(ExerciseType et: ExerciseType.values()) {
			et.getExternalName();
		}
		
		for(GitProjectVisibility vis: GitProjectVisibility.values()) {
			vis.getExternalName();
		}
	}

	// @org.junit.jupiter.api.Test
	// public void testSearchInputDTO() throws IllegalAccessException, InvocationTargetException {
	// 	propertiesTester.testProperties(SearchInputDTO.class);
	// }
	
	@org.junit.jupiter.api.Test
	public void testDeploymentInfo() throws IllegalAccessException, InvocationTargetException {
		propertiesTester.testProperties(DeploymentInfo.class);
	}

	@org.junit.jupiter.api.Test
	public void testVariousEquals() throws IllegalAccessException, InvocationTargetException {
		GitProject p1 = new GitProject(); p1.setProjectId(10);
		GitProject p2 = new GitProject(); p2.setProjectId(10);
		GitProject p3 = new GitProject(); p3.setProjectId(20);

		Assert.assertEquals(p1, p2);
		Assert.assertNotEquals(p1, p3);
		EqualsVerifier.forClass(GitProject.class)
		    .suppress(Warning.NONFINAL_FIELDS)
		    .withIgnoredFields("project_name", "namespace","main_group", "last_activity_at", "url", "sub_group").verify();
		EqualsVerifier.forClass(Person.class).suppress(Warning.NONFINAL_FIELDS).verify();
	}

}
